@@ -24,7 +24,8 @@ from ldap3 import ALL_ATTRIBUTES, ASYNC, AUTO_BIND_DEFAULT, AUTO_BIND_NONE, BASE
 from ldap3.core.exceptions import LDAPBindError
 from ldap3.utils.conv import escape_filter_chars
 from persistent import Persistent
-from pyams_auth_ldap.interfaces import ILDAPGroupInfo, ILDAPPlugin, ILDAPUserInfo
+from pyams_auth_ldap.interfaces import ILDAPGroupInfo, ILDAPPlugin, ILDAPUserInfo, \
+    INTERNAL_GROUP_MAIL_MODE, NO_GROUP_MAIL_MODE, QUERY_MEMBERS_FROM_GROUP
 from pyams_auth_ldap.query import LDAPQuery
 from zope.container.contained import Contained
 from zope.interface import implementer
@@ -43,47 +44,19 @@ __docformat__ = 'restructuredtext'
 
 LOGGER = logging.getLogger('PyAMS (ldap)')
 
-
-managers = {}
-
 FORMAT_ATTRIBUTES = re.compile("\{(\w+)\[?\d*\]?\}")
 
+DN_ATTRIBUTE = 'dn'
+ANY_OBJECT_FILTER = '(objectClass=*)'
+GROUP_ATTR_FILTER = '({attribute}={{group_dn}})'
 
-class ConnectionManager:
-    """LDAP connections manager"""
+USER_DN_PREFIX = '{prefix}:{dn}'
+USER_ATTR_PREFIX = '{prefix}:{attr}'
+GROUP_DN_PREFIX = '{prefix}:{group_prefix}:{dn}'
+GROUP_ATTR_PREFIX = '{prefix}:{group_prefix}:{attr}'
 
-    _connection = None
-
-    def __init__(self, plugin):
-        self.server = Server(plugin.host,
-                             port=plugin.port,
-                             use_ssl=plugin.use_ssl)
-        self.bind_dn = plugin.bind_dn
-        self.password = plugin.bind_password
-
-    def get_connection(self, user=None, password=None, read_only=True):
-        """Open LDAP connection"""
-        if user:
-            conn = Connection(self.server,
-                              user=user, password=password,
-                              client_strategy=ASYNC,
-                              auto_bind=True,
-                              lazy=False,
-                              read_only=read_only)
-        else:
-            conn = self._connection
-            if conn is None:
-                bind_mode = AUTO_BIND_DEFAULT if self.bind_dn else AUTO_BIND_NONE
-                conn = Connection(self.server,
-                                  user=self.bind_dn, password=self.password,
-                                  client_strategy=REUSABLE,
-                                  auto_bind=bind_mode,
-                                  lazy=True,
-                                  read_only=read_only)
-                if conn.auto_bind == AUTO_BIND_NONE:
-                    conn.open(read_server_info=False)
-                self._connection = conn
-        return conn
+LOGIN_PREFIX = '{prefix}:{login}'
+GROUP_ID_PREFIX = '{prefix}:{group_prefix}:{group_id}'
 
 
 @implementer(ILDAPUserInfo)
@@ -136,7 +109,7 @@ class LDAPGroupMailInfoAdapter(ContextAdapter):
         group = self.context
         plugin = group.plugin
 
-        if plugin.group_mail_mode == 'none':
+        if plugin.group_mail_mode == NO_GROUP_MAIL_MODE:
             # use members address
             for member in plugin.get_members(group, info=False):
                 mail_info = IPrincipalMailInfo(member, None)
@@ -144,21 +117,21 @@ class LDAPGroupMailInfoAdapter(ContextAdapter):
                     for address in mail_info.get_addresses():
                         yield address
 
-        elif plugin.group_mail_mode == 'internal':
+        elif plugin.group_mail_mode == INTERNAL_GROUP_MAIL_MODE:
             # use group internal attribute
             mail = group.attributes.get(plugin.group_mail_attribute)
             if mail:
                 yield plugin.group_title_format(**group.attributes), \
                       mail[0] if isinstance(mail, (list, tuple)) else mail
 
-        else:
+        else:  # REDIRECT_GROUP_MAIL_MODE
             # redirect: use internal attribute of another group
             source, target = plugin.group_replace_expression.split('|')
             target_dn = group.dn.replace(source, target)
             conn = plugin.get_connection()
             attributes = FORMAT_ATTRIBUTES.findall(plugin.group_title_format) + \
                          [plugin.group_mail_attribute]
-            search = LDAPQuery(target_dn, '(objectClass=*)', BASE, attributes)
+            search = LDAPQuery(target_dn, ANY_OBJECT_FILTER, BASE, attributes)
             result = search.execute(conn)
             if not result or len(result) > 1:
                 raise StopIteration
@@ -169,6 +142,50 @@ class LDAPGroupMailInfoAdapter(ContextAdapter):
                       mail[0] if isinstance(mail, (list, tuple)) else mail
 
 
+#
+# LDAP connections managers
+#
+
+LDAP_MANAGERS = {}
+
+
+class ConnectionManager:
+    """LDAP connections manager"""
+
+    _connection = None
+
+    def __init__(self, plugin):
+        self.server = Server(plugin.host,
+                             port=plugin.port,
+                             use_ssl=plugin.use_ssl)
+        self.bind_dn = plugin.bind_dn
+        self.password = plugin.bind_password
+
+    def get_connection(self, user=None, password=None, read_only=True):
+        """Open LDAP connection"""
+        if user:
+            conn = Connection(self.server,
+                              user=user, password=password,
+                              client_strategy=ASYNC,
+                              auto_bind=True,
+                              lazy=False,
+                              read_only=read_only)
+        else:
+            conn = self._connection
+            if conn is None:
+                bind_mode = AUTO_BIND_DEFAULT if self.bind_dn else AUTO_BIND_NONE
+                conn = Connection(self.server,
+                                  user=self.bind_dn, password=self.password,
+                                  client_strategy=REUSABLE,
+                                  auto_bind=bind_mode,
+                                  lazy=True,
+                                  read_only=read_only)
+                if conn.auto_bind == AUTO_BIND_NONE:
+                    conn.open(read_server_info=False)
+                self._connection = conn
+        return conn
+
+
 @factory_config(ILDAPPlugin)
 class LDAPPlugin(Persistent, Contained):
     """LDAP authentication plug-in"""
@@ -260,14 +277,14 @@ class LDAPPlugin(Persistent, Contained):
 
     def clear(self):
         self_id = self._get_id()
-        if self_id in managers:
-            del managers[self_id]
+        if self_id in LDAP_MANAGERS:
+            del LDAP_MANAGERS[self_id]
 
     def get_connection(self, user=None, password=None):
         self_id = self._get_id()
-        if self_id not in managers:
-            managers[self_id] = ConnectionManager(self)
-        connection = managers[self_id].get_connection(user, password)
+        if self_id not in LDAP_MANAGERS:
+            LDAP_MANAGERS[self_id] = ConnectionManager(self)
+        connection = LDAP_MANAGERS[self_id].get_connection(user, password)
         if connection.closed:
             connection.open(read_server_info=False)
         return connection
@@ -293,13 +310,13 @@ class LDAPPlugin(Persistent, Contained):
             LOGGER.debug("LDAP authentication exception with login %r", login, exc_info=True)
             return None
         else:
-            if self.uid_attribute == 'dn':
-                return "{prefix}:{dn}".format(prefix=self.prefix,
-                                              dn=login_dn)
+            if self.uid_attribute == DN_ATTRIBUTE:
+                return USER_DN_PREFIX.format(prefix=self.prefix,
+                                             dn=login_dn)
             attrs = result[1]
             if self.login_attribute in attrs:
-                return "{prefix}:{attr}".format(prefix=self.prefix,
-                                                attr=attrs[self.uid_attribute][0])
+                return USER_ATTR_PREFIX.format(prefix=self.prefix,
+                                               attr=attrs[self.uid_attribute][0])
             return None
 
     def _get_group(self, group_id):
@@ -319,8 +336,8 @@ class LDAPPlugin(Persistent, Contained):
                          [self.group_mail_attribute]
             if self.group_extra_attributes:
                 attributes += self.group_extra_attributes.split(',')
-            if self.group_uid_attribute == 'dn':
-                search = LDAPQuery(group_id, '(objectClass=*)', BASE, attributes)
+            if self.group_uid_attribute == DN_ATTRIBUTE:
+                search = LDAPQuery(group_id, ANY_OBJECT_FILTER, BASE, attributes)
             else:
                 search = LDAPQuery(self.base_dn, self.uid_query, self.search_scope, attributes)
             result = search.execute(conn, login=group_id)
@@ -328,14 +345,14 @@ class LDAPPlugin(Persistent, Contained):
                 return None
             group_dn, attrs = result[0]
             if info:
-                return PrincipalInfo(id='{prefix}:{group_prefix}:{group_id}'.format(
+                return PrincipalInfo(id=GROUP_ID_PREFIX.format(
                     prefix=self.prefix,
                     group_prefix=self.group_prefix,
                     group_id=group_id),
                                      title=self.group_title_format.format(**attrs),
                                      dn=group_dn)
             else:
-                attrs.update({'principal_id': '{prefix}:{group_prefix}:{group_id}'.format(
+                attrs.update({'principal_id': GROUP_ID_PREFIX.format(
                     prefix=self.prefix,
                     group_prefix=self.group_prefix,
                     group_id=group_id)})
@@ -344,8 +361,8 @@ class LDAPPlugin(Persistent, Contained):
             attributes = FORMAT_ATTRIBUTES.findall(self.title_format) + [self.mail_attribute]
             if self.user_extra_attributes:
                 attributes += self.user_extra_attributes.split(',')
-            if self.uid_attribute == 'dn':
-                search = LDAPQuery(login, '(objectClass=*)', BASE, attributes)
+            if self.uid_attribute == DN_ATTRIBUTE:
+                search = LDAPQuery(login, ANY_OBJECT_FILTER, BASE, attributes)
             else:
                 search = LDAPQuery(self.base_dn, self.uid_query, self.search_scope, attributes)
             result = search.execute(conn, login=login)
@@ -353,20 +370,20 @@ class LDAPPlugin(Persistent, Contained):
                 return None
             user_dn, attrs = result[0]
             if info:
-                return PrincipalInfo(id='{prefix}:{login}'.format(prefix=self.prefix,
-                                                                  login=login),
+                return PrincipalInfo(id=LOGIN_PREFIX.format(prefix=self.prefix,
+                                                            login=login),
                                      title=self.title_format.format(**attrs),
                                      dn=user_dn)
-            attrs.update({'principal_id': '{prefix}:{login}'.format(
+            attrs.update({'principal_id': LOGIN_PREFIX.format(
                 prefix=self.prefix, login=login)})
             return LDAPUserInfo(user_dn, attrs, self)
 
     def _get_groups(self, principal):
         """Get principal groups"""
-        principal_dn = principal.attributes.get('dn')
+        principal_dn = principal.attributes.get(DN_ATTRIBUTE)
         if principal_dn is None:
             raise StopIteration
-        if self.group_members_query_mode == 'group':
+        if self.group_members_query_mode == QUERY_MEMBERS_FROM_GROUP:
             # group members are defined inside group
             if not self.groups_base_dn:
                 raise StopIteration
@@ -375,35 +392,35 @@ class LDAPPlugin(Persistent, Contained):
             search = LDAPQuery(self.groups_base_dn, self.groups_query, self.groups_search_scope,
                                attributes)
             for group_dn, group_attrs in search.execute(conn, dn=principal_dn):
-                if self.group_uid_attribute == 'dn':
-                    yield '{prefix}:{group_prefix}:{dn}'.format(
+                if self.group_uid_attribute == DN_ATTRIBUTE:
+                    yield GROUP_DN_PREFIX.format(
                         prefix=self.prefix,
                         group_prefix=self.group_prefix,
                         dn=group_dn)
                 else:
-                    yield '{prefix}:{group_prefix}:{attr}'.format(
+                    yield GROUP_ATTR_PREFIX.format(
                         prefix=self.prefix,
                         group_prefix=self.group_prefix,
                         attr=group_attrs[self.group_uid_attribute])
-        else:
+        else:  # QUERY_MEMBERS_FROM_MEMBER
             # a member defines it's groups
             conn = self.get_connection()
             attributes = [self.user_groups_attribute]
-            user_search = LDAPQuery(principal_dn, '(objectClass=*)', BASE, attributes)
+            user_search = LDAPQuery(principal_dn, ANY_OBJECT_FILTER, BASE, attributes)
             for user_dn, user_attrs in user_search.execute(conn):
-                if self.group_uid_attribute == 'dn':
+                if self.group_uid_attribute == DN_ATTRIBUTE:
                     for group_dn in user_attrs.get(self.user_groups_attribute, ()):
-                        yield '{prefix}:{group_prefix}:{dn}'.format(
+                        yield GROUP_DN_PREFIX.format(
                             prefix=self.prefix,
                             group_prefix=self.group_prefix,
                             dn=group_dn)
                 else:
                     attributes = [self.group_uid_attribute]
                     for group_dn in user_attrs.get(self.user_groups_attribute, ()):
-                        group_search = LDAPQuery(group_dn, '(objectClass=*)', BASE,
+                        group_search = LDAPQuery(group_dn, ANY_OBJECT_FILTER, BASE,
                                                  attributes)
                         for group_search_dn, group_search_attrs in group_search.execute(conn):
-                            yield '{prefix}:{group_prefix}:{attr}'.format(
+                            yield GROUP_ATTR_PREFIX.format(
                                 prefix=self.prefix,
                                 group_prefix=self.group_prefix,
                                 attr=group_search_attrs[self.group_uid_attribute])
@@ -426,42 +443,42 @@ class LDAPPlugin(Persistent, Contained):
         if not self.enabled:
             return set()
         conn = self.get_connection()
-        if self.group_members_query_mode == 'group':
+        if self.group_members_query_mode == QUERY_MEMBERS_FROM_GROUP:
             # group members are defined into group attribute
             attributes = [self.group_members_attribute]
             user_attributes = FORMAT_ATTRIBUTES.findall(self.title_format) + \
                               [self.mail_attribute]
-            search = LDAPQuery(group.dn, '(objectClass=*)', BASE, attributes)
+            search = LDAPQuery(group.dn, ANY_OBJECT_FILTER, BASE, attributes)
             for group_dn, attrs in search.execute(conn):
                 for user_dn in attrs.get(self.group_members_attribute):
-                    user_search = LDAPQuery(user_dn, '(objectClass=*)', BASE, user_attributes)
+                    user_search = LDAPQuery(user_dn, ANY_OBJECT_FILTER, BASE, user_attributes)
                     for user_search_dn, user_search_attrs in user_search.execute(conn):
                         if info:
-                            yield PrincipalInfo(id='{prefix}:{dn}'.format(prefix=self.prefix,
-                                                                          dn=user_search_dn),
+                            yield PrincipalInfo(id=USER_DN_PREFIX.format(prefix=self.prefix,
+                                                                         dn=user_search_dn),
                                                 title=self.title_format.format(
                                                     **user_search_attrs),
                                                 dn=user_search_dn)
                         else:
                             yield LDAPUserInfo(dn=user_search_dn, attributes=user_search_attrs,
                                                plugin=self)
-        else:
+        else:  # QUERY_MEMBERS_FROM_MEMBER
             # member groups are defined into member attribute
             attributes = FORMAT_ATTRIBUTES.findall(self.title_format) + \
                          [self.uid_attribute, self.mail_attribute]
-            search = LDAPQuery(self.base_dn, '({attribute}={{group_dn}})'.format(
+            search = LDAPQuery(self.base_dn, GROUP_ATTR_FILTER.format(
                 attribute=self.user_groups_attribute),
                                self.search_scope, attributes)
             for user_dn, user_attrs in search.execute(conn, group_dn=group.dn):
                 if info:
-                    if self.uid_attribute == 'dn':
-                        yield PrincipalInfo(id='{prefix}:{dn}'.format(
+                    if self.uid_attribute == DN_ATTRIBUTE:
+                        yield PrincipalInfo(id=USER_DN_PREFIX.format(
                             prefix=self.prefix,
                             dn=user_dn),
                                             title=self.title_format.format(**user_attrs),
                                             dn=user_dn)
                     else:
-                        yield PrincipalInfo(id='{prefix}:{attr}'.format(
+                        yield PrincipalInfo(id=USER_ATTR_PREFIX.format(
                             prefix=self.prefix,
                             attr=user_attrs[self.uid_attribute][0]),
                                             title=self.title_format.format(**user_attrs),
@@ -480,13 +497,13 @@ class LDAPPlugin(Persistent, Contained):
         search = LDAPQuery(self.base_dn, self.users_select_query,
                            self.search_scope, attributes)
         for user_dn, user_attrs in search.execute(conn, query=query):
-            if self.uid_attribute == 'dn':
-                yield PrincipalInfo(id='{prefix}:{dn}'.format(prefix=self.prefix,
-                                                              dn=user_dn),
+            if self.uid_attribute == DN_ATTRIBUTE:
+                yield PrincipalInfo(id=USER_DN_PREFIX.format(prefix=self.prefix,
+                                                             dn=user_dn),
                                     title=self.title_format.format(**user_attrs),
                                     dn=user_dn)
             else:
-                yield PrincipalInfo(id='{prefix}:{attr}'.format(
+                yield PrincipalInfo(id=USER_ATTR_PREFIX.format(
                     prefix=self.prefix,
                     attr=user_attrs[self.uid_attribute][0]),
                                     title=self.title_format.format(**user_attrs),
@@ -498,15 +515,15 @@ class LDAPPlugin(Persistent, Contained):
             search = LDAPQuery(self.groups_base_dn, self.groups_select_query,
                                self.groups_search_scope, attributes)
             for group_dn, group_attrs in search.execute(conn, query=query):
-                if self.group_uid_attribute == 'dn':
-                    yield PrincipalInfo(id='{prefix}:{group_prefix}:{dn}'.format(
+                if self.group_uid_attribute == DN_ATTRIBUTE:
+                    yield PrincipalInfo(id=GROUP_DN_PREFIX.format(
                         prefix=self.prefix,
                         group_prefix=self.group_prefix,
                         dn=group_dn),
                                         title=self.group_title_format.format(**group_attrs),
                                         dn=group_dn)
                 else:
-                    yield PrincipalInfo(id='{prefix}:{group_prefix}:{attr}'.format(
+                    yield PrincipalInfo(id=GROUP_ATTR_PREFIX.format(
                         prefix=self.prefix,
                         group_prefix=self.group_prefix,
                         attr=group_attrs[self.group_uid_attribute][0]),
